;--
;
; This code supports the PIC18F2620 POV project.
;
; This top level module initializes the chip by a combination of processor
; configuration settings and cpu writes to multiple registers.
;
; In summary - it will translate an ASCII string stored in flash memory into
; a series of bits that active the LEDS at each pixel clock time.
;
; Two CPU timers are used. One to measure the rotational speed and one to
; generate the display clock. The code has been hand tuned to work with
; a motor speed of approximately 1800 RPM or 30msec per revolution.
;
; An IR emitter / detector senses the presence of a white strip on a black
; background that is used to indicate the rotation index or time zero. The
; detector causes an interrupt. Within the ISR, the code compute the new pixel
; clock and enables the second counter.
;
; Note - It is important that the timer does not overflow. 
;
; The pixel clock interrupts the CPU approximately every 250 usec or 200 times
; per revolution. Inside the ISR the processor output the next 8 data bits to
; drive the LEDS. This scheme supports a display message of 28 characters at
; 7 columns per character.
;
; The LEDs are active low which can cause some confusion when trying to follow
; the code.
;
; Currently the upper 2 LEDs are not used. Initail intenet was to support
; larger characters.
;
; Copyright (c) 2015 George M. Gallant, Jr.
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in
; all copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
; THE SOFTWARE.
;
;--
		include	"pov.inc"

		extern	fontTbl
		extern	setFntPtr
		extern	tmr2Init, tmr3Init, ext2Init
		extern	hpIsr, lpIsr

		global	msgBuf
		
		CONFIG WDT=OFF				; disable watchdog timer
		CONFIG MCLRE = ON			; MCLEAR Pin on
		CONFIG DEBUG = ON			; Enable Debug Mode
		CONFIG LVP = OFF			; Low-Voltage programming disabled (necessary for debugging)
		CONFIG OSC = INTIO67			; Internal oscillator, port function on RA6, RA7

		udata_acs

	
bitMapData	udata	0x200

msgBuf:		res	1				; this is just a marker byte

textData	udata	0x100
		
textBuf:	res	16
	
		org	0x0000

		goto	sys_init

;	High Priority Interrupt Vector

		org	0x0008

		goto	hpIsr

;	Low Priority Interrupt Vector

		org	0x0018

		goto	lpIsr

		code
;--
;
;	System Initialization
;
;--
sys_init:	clrf	LATA				; initialize the ports
		clrf	LATB
		clrf	LATC

		movlw	0xFF				; make all ports digital
		movwf	ADCON1
		
		clrf	RCON
		bsf	RCON, IPEN			; enable priority interrupts
		clrf	INTCON
		bsf	INTCON2, RBPU

		movlw	(1 << IDLEN) | (7 << 4)
		movwf	OSCCON

		movlw	0xFF				; PORTA - IIII IIII
		movwf	TRISA

		movlw	0xFC				; PORTB - IIII IIOO
		movwf	TRISB

		movlw	0x00				; PORTC - OOOO OOOO
		movwf	TRISC

		movlb	1				; setup the BSR

		movlw	2				; setup a stack
		movwf	STKPTR

		clrf	PCLATH

		movlw	0x40				; PLLEN
		movwf	OSCTUNE

		push
		
		movlw	3
		movwf	LATB
		movlw	0xFF
		movwf	LATC
		
;	 Clear the entire RAM. Wipe all banks

		lfsr	0,0
		movlw	15
@si1:		clrf	POSTINC0
		cpfseq	FSR0H
		bra	@si1

		rcall	genMessage
		
		rcall	tmr2Init
		rcall	tmr3Init
		rcall	ext2Init			; enable interrputs on RB2 - EXT2
		
;	Enable Interrupts

		bsf	INTCON, GIEH			; enable interrupts
		bsf	INTCON, GIEL
		
;	Loop here doing nothing. All work is done in the ISRs
		
@si2:		nop
		bra	@si2

;	Text Message must be even number of characters
		
;			"123456789012345678902345678"
textMsg:	db	"Let's Make Robots", 0

;--
;
;	Copy message from FLASH to RAM. Might be unnecessary
;
;--
genMessage:	push

;	Set all display memory to 0xFF (blank)
		
		clrf	TOSL
		movlw	high(msgBuf)
		movwf	FSR0H
		movlw	low(msgBuf)
		movwf	FSR0L
		movlw	0xFF
@gm0:		movwf	POSTINC0
		decfsz	TOSL, f
		bra	@gm0
		
;	Copy ascii message from flash to RAM. Leave char count in TOSL
	
		movlw	upper(textMsg)
		movwf	TBLPTRU
		movlw	high(textMsg)
		movwf	TBLPTRH
		movlw	low(textMsg)
		movwf	TBLPTRL

		movlw	high(textBuf)
		movwf	FSR1H
		movlw	low(textBuf)
		movwf	FSR1L
		
		clrf	TOSL
@gm1:		tblrd	*+				; get char from message
		movf	TABLAT, w
		bz	@gm2
		movwf	POSTINC1			; park in memory
		incf	TOSL, f				; bump the char count
		bra	@gm1
		
;	Generate the raster image in RAM

@gm2:		movlw	high(msgBuf)
		movwf	FSR0H
		movlw	low(msgBuf)
		movwf	FSR0L
		
;	Compute the starting byte position
		
		movlw	101				; mid point
		movwf	TOSH
		
		movlw	7				; pixel colums per display frame
		mulwf	TOSL				; number of chars
		
		bcf	STATUS, C
		rrcf	PRODL, w
		subwf	TOSH, w
		addwf	FSR0L, f
		
		movlw	high(textBuf)
		movwf	FSR1H
		movlw	low(textBuf)
		movwf	FSR1L

@gm3:		movlw	NUM_COL				; number of columns per char
		movwf	TOSH

		movf	POSTINC1, w
		nop
		nop
		rcall	setFntPtr

@gm4:		tblrd	*+				; get column data
		movf	TABLAT, w
		nop
		nop
		comf	WREG
		movwf	POSTINC0			; store in memory
		decfsz	TOSH, f
		bra	@gm4

		decfsz	TOSL, f
		bra	@gm3
		
		pop
		return
		
		end
